@@ -238,7 +238,7 @@ import math
 from math import sqrt, log  # Optimization: no attribute look-up
 import copy
 
-__version__ = '1.5.4.3'
+__version__ = '1.5.4.4'
 # Numerical version:
 __version_info__ = tuple(int(num) for num in __version__.split('.'))
 
@@ -696,17 +696,16 @@ class AffineScalarFunc(object):
 
     def __init__(self, nominal_value, derivatives):
         """
-        'nominal_value' is the value of the function at the origin.
-        
-        'derivatives' maps each Variable object on which the function
+        nominal_value -- value of the function at the origin.
+        nominal_value must not depend in any way of the Variable
+        objects in 'derivatives' (the value at the origin of the
+        function being defined is a constant).
+
+        derivatives -- maps each Variable object on which the function
         being defined depends to the value of the derivative with
         respect to that variable, taken at the nominal value of all
         variables.
  
-        'nominal_value' must not depend in any way of the Variable
-        objects in 'derivatives' (the value at the origin of the
-        function being defined is a constant).
-
         Warning: the above constraint is not checked, and the user is
         responsible for complying with it.
         """
@@ -722,6 +721,8 @@ class AffineScalarFunc(object):
         self._nominal_value = float(nominal_value)
         self.derivatives = derivatives
 
+    # The following prevents the 'nominal_value' attribute from being
+    # modified by the user:
     @property
     def nominal_value(self):
         "Nominal value of the random number."
@@ -888,8 +889,24 @@ class AffineScalarFunc(object):
             raise ValueError("The standard deviation is zero:"
                              " undefined result.")
 
-    # Hooks for the pickle module:
-    def __getstate__(self):        
+    def __deepcopy__(self, memo):
+        """
+        Hook for the standard copy module.
+
+        The returned AffineScalarFunc is a completely fresh copy,
+        which is fully independent of any variable defined so far.
+        New variables are specially created for the returned
+        AffineScalarFunc object.
+        """
+        return AffineScalarFunc(
+            self._nominal_value,
+            dict((copy.deepcopy(var), deriv)
+                 for (var, deriv) in self.derivatives.iteritems()))
+
+    def __getstate__(self):
+        """
+        Hook for the pickle module.
+        """
         obj_slot_values = dict((k, getattr(self, k)) for k in
                                # self.__slots__ would not work when
                                # self is an instance of a subclass:
@@ -897,6 +914,9 @@ class AffineScalarFunc(object):
         return obj_slot_values
 
     def __setstate__(self, data_dict):
+        """
+        Hook for the pickle module.
+        """        
         for (name, value) in data_dict.iteritems():
             setattr(self, name, value)
 
@@ -1058,6 +1078,9 @@ class Variable(AffineScalarFunc):
         # takes much more memory.  Thus, this implementation chooses
         # more cycles and a smaller memory footprint instead of no
         # cycles and a larger memory footprint.
+
+        # ! Using AffineScalarFunc instead of super() results only in
+        # a 3 % speed loss (Python 2.6, Mac OS X):
         super(Variable, self).__init__(value, {self: 1.})
 
         # We force the error to be float-like.  Since it is considered
@@ -1126,6 +1149,8 @@ class Variable(AffineScalarFunc):
     def __deepcopy__(self, memo):
         """
         Hook for the standard copy module.
+
+        A new variable is created.
         """
         
         # This deep copy implicitly takes care of the reference of the
@@ -1134,21 +1159,21 @@ class Variable(AffineScalarFunc):
 
         # Reference: http://www.doughellmann.com/PyMOTW/copy/index.html
         
-        # For robustness reasons, parameters are deep-copied (in case
-        # Variable objects contain, in some version, attributes that
-        # are not simple floats):
-        return Variable(copy.deepcopy(self.nominal_value, memo),
-                        copy.deepcopy(self.std_dev(), memo),
-                        copy.deepcopy(self.tag, memo))
+        return self.__copy__()
 
-    # Hooks for the pickle module:
-    def __getstate__(self):        
+    def __getstate__(self):
+        """
+        Hook for the standard pickle module.
+        """
         obj_slot_values = dict((k, getattr(self, k)) for k in self.__slots__)
         obj_slot_values.update(AffineScalarFunc.__getstate__(self))
         # Conversion to a usual dictionary:
         return obj_slot_values
 
     def __setstate__(self, data_dict):
+        """
+        Hook for the standard pickle module.
+        """        
         for (name, value) in data_dict.iteritems():
             setattr(self, name, value)
         
